"""
Configs migration

Generation datetime: 2021-02-01 10:36:35.624326
"""

from typing import List

from configs_core.migration.db_migrate_helper import DatabaseMigrationHelper as mh, LimitationToModify

REVISION = "f42525e2"
DOWN_REVISION = "8bd6e404"


def upgradeFaceDetectorV3Setting(setting: dict) -> dict:
    """Upgrade FaceDetectorV3 setting."""
    setting.pop("use_orientation_mode", None)
    return setting


def downgradeFaceDetectorV3Setting(setting: dict) -> dict:
    """Downgrade FaceDetectorV3 setting."""
    setting["use_orientation_mode"] = False
    return setting


limitationsToModify: List[LimitationToModify] = [
    LimitationToModify(
        limitationName="FACE_DETECTOR_V3",
        oldValidationSchema={
            "type": "object",
            "properties": {
                "min_face_size": {"type": "integer", "minimum": 1},
                "max_face_size": {"type": "integer", "minimum": 1},
                "redetect_tensor_size": {"type": "integer", "minimum": 1},
                "redetect_face_target_size": {"type": "integer", "minimum": 1},
                "redetect_score_threshold": {"type": "number"},
                "score_threshold": {"type": "number"},
                "use_orientation_mode": {"type": "boolean"},
            },
            "description": "settings for face detector version 3",
            "additionalProperties": False,
        },
        newValidationSchema={
            "type": "object",
            "properties": {
                "min_face_size": {"type": "integer", "minimum": 1},
                "max_face_size": {"type": "integer", "minimum": 1},
                "redetect_tensor_size": {"type": "integer", "minimum": 1},
                "redetect_face_target_size": {"type": "integer", "minimum": 1},
                "redetect_score_threshold": {"type": "number"},
                "score_threshold": {"type": "number"},
            },
            "description": "settings for face detector version 3",
            "additionalProperties": False,
        },
        oldDefaultValue={
            "max_face_size": 640,
            "min_face_size": 50,
            "redetect_tensor_size": 80,
            "redetect_face_target_size": 45,
            "redetect_score_threshold": 0.357,
            "score_threshold": 0.89,
            "use_orientation_mode": False,
        },
        newDefaultValue={
            "max_face_size": 640,
            "min_face_size": 50,
            "redetect_tensor_size": 80,
            "redetect_face_target_size": 45,
            "redetect_score_threshold": 0.357,
            "score_threshold": 0.89,
        },
        settingUpgradeFunction=upgradeFaceDetectorV3Setting,
        settingDowngradeFunction=downgradeFaceDetectorV3Setting,
        # DON'T REMOVE SERVICES. Used in profiled migrations.
        oldServices=["luna-handlers"],
        newServices=["luna-handlers"],
    ),
]


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_HANDLERS_USE_AUTO_ROTATION",
        validationSchema={"type": "boolean"},
        defaultValue=False,
        description="whether to try estimate image rotation angle and rotate to normal state",
        services=["luna-handlers"],
        groupNames=["luna-platform", "luna-api sdk settings"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_HANDLERS_USE_AUTO_ROTATION")
