"""
Configs migration

Generation datetime: 2024-05-23 12:54:51.907357
"""

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)

mh: DatabaseMigrationHelper

REVISION = "94742577"
DOWN_REVISION = "73a100fe"


limitationsToModify: list[LimitationToModify] = []

externalLimitationsToModify: list[ExternalLimitationToModify] = []


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_LAMBDA_BUILD_LIMITS",
        validationSchema={
            "type": "object",
            "properties": {
                "cpu_limit": {"type": "integer", "minimum": 1},
                "ram_limit": {"type": "integer", "minimum": 1},
                "cpu_request": {"type": "integer", "minimum": 1},
                "ram_request": {"type": "number", "exclusiveMinimum": 0},
            },
            "additionalProperties": False,
        },
        defaultValue={
            "cpu_limit": 2000,
            "ram_limit": 4,
            "cpu_request": 500,
            "ram_request": 0.5,
        },
        description="Resource requests and limits for lambda build.",
        services=["luna-lambda"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_LAMBDA_BUILD_LIMITS")
