"""
Module contains settings class for luna-backport4
"""

from typing import Optional

from ..settings.classes import (
    RedisKeyStorageSetting as _RedisKeyStorageSetting,
    ServiceAddressSettings as _ServiceAddressSettings,
    ServiceTimeoutsSettings as _ServiceTimeoutsSettings,
)
from .base_python_service_settings import BasePythonServiceSettingsClass


class SettingsBackport4(BasePythonServiceSettingsClass):
    """
    Settings class for luna-backport4
    Attributes:
        apiAddress(ServiceAddressSettings): luna-api address settings
        apiTimeouts(ServiceAddressSettings): luna-api timeouts settings
        facesAddress(ServiceAddressSettings): luna-faces address settings
        facesTimeouts(ServiceTimeoutsSettings): luna-faces timeouts settings
        attributesStorageSettings (RedisStorageSetting): temporary attributes storage settings
    """

    serviceName = "luna-backport4"

    def __init__(self):
        super().__init__()

        self.apiAddress: Optional[_ServiceAddressSettings] = None
        self.apiTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.facesAddress: Optional[_ServiceAddressSettings] = None
        self.facesTimeouts: Optional[_ServiceTimeoutsSettings] = None
        self.attributesStorageSettings: Optional[_RedisKeyStorageSetting] = None

    def applySettings(self):
        """
        Deserialize loaded settings to classes
        """
        super().applySettings()
        self.apiAddress = _ServiceAddressSettings(self.serviceName, "LUNA_API_ADDRESS", self["LUNA_API_ADDRESS"])
        self.apiTimeouts = _ServiceTimeoutsSettings(self.serviceName, "LUNA_API_TIMEOUTS", self["LUNA_API_TIMEOUTS"])
        self.facesAddress = _ServiceAddressSettings(self.serviceName, "LUNA_FACES_ADDRESS", self["LUNA_FACES_ADDRESS"])
        self.facesTimeouts = _ServiceTimeoutsSettings(
            self.serviceName, "LUNA_FACES_TIMEOUTS", self["LUNA_FACES_TIMEOUTS"]
        )
        self.attributesStorageSettings: Optional[_RedisKeyStorageSetting] = _RedisKeyStorageSetting(
            self.serviceName, "ATTRIBUTES_STORAGE_POLICY", self["ATTRIBUTES_STORAGE_POLICY"]
        )
