from configs_core.base_classes import BaseSettingClass
from configs_core.base_setting_definition import BaseSettingDefinition
from fastjsonschema import validate

from configs.services.base_python_service_settings import BasePythonServiceSettingsClass
from configs.settings import schemas

TEST_VALUES = {str: "test", int: 123, float: 0.123, bool: True}
DEFAULT_VALUES = {str: "default", int: 777, float: 0.777, bool: False}
assert all((TEST_VALUES[k] != DEFAULT_VALUES[k] for k in TEST_VALUES))
# =============== File Configs =========================================================================================
fileConf = f"""
[LUNA_CONFIGURATOR]
USE_CONFIGURATOR = 0
LUNA_CONFIGURATOR_ORIGIN = http://127.0.0.1:5070
LUNA_CONFIGURATOR_API  = 1

[LARGE_NESTED_SETTING]
SOME_ROOT_FIELD = {TEST_VALUES[str]}

[LARGE_NESTED_SETTING.SOME_DATA]
SOME_STR = {TEST_VALUES[str]}
SOME_INT = {TEST_VALUES[int]}
SOME_FLOAT = {TEST_VALUES[float]}
SOME_BOOL = {int(TEST_VALUES[bool])}
SOME_EMPTY_LIST = [] 
SOME_STR_LIST = [{TEST_VALUES[str]}, {TEST_VALUES[str]},{TEST_VALUES[str]}]  
SOME_INT_LIST = [{TEST_VALUES[int]}, {TEST_VALUES[int]},{TEST_VALUES[int]}]  
SOME_FLOAT_LIST = [{TEST_VALUES[float]}, {TEST_VALUES[float]},{TEST_VALUES[float]}]  
SOME_BOOL_LIST = [{int(TEST_VALUES[bool])}, {int(TEST_VALUES[bool])},{int(TEST_VALUES[bool])}]  

[LARGE_NESTED_SETTING.SOME_DATA.SOME_EMPTY_DICT]

[LARGE_NESTED_SETTING.SOME_DATA.SOME_FULL_DICT]
NAME = {TEST_VALUES[str]}

[LARGE_NESTED_SETTING.SOME_DATA.SOME_FULL_LIST.0]

[LARGE_NESTED_SETTING.SOME_DATA.SOME_FULL_LIST.1]
NAME = {TEST_VALUES[str]}
"""
minimalFileConf = ""
secondOneOfFileConf = """
[LARGE_NESTED_SETTING]
IMPORT = this
"""
impossibleOneOfFileConf = f"""
[LARGE_NESTED_SETTING]
#: from first oneOf
SOME_ROOT_FIELD = {TEST_VALUES[str]}
#: from second oneOf
IMPORT = this
"""

# ============= Puller Configs =========================================================================================
pullerConf = {
    "LARGE_NESTED_SETTING": {
        "value": {
            "some_root_field": TEST_VALUES[str],
            "some_data": {
                "some_str": TEST_VALUES[str],
                "some_int": TEST_VALUES[int],
                "some_float": TEST_VALUES[float],
                "some_bool": TEST_VALUES[bool],
                "some_empty_list": [],
                "some_str_list": [TEST_VALUES[str]] * 3,
                "some_int_list": [TEST_VALUES[int]] * 3,
                "some_float_list": [TEST_VALUES[float]] * 3,
                "some_bool_list": [TEST_VALUES[bool]] * 3,
                "some_empty_dict": {},
                "some_full_dict": {"name": TEST_VALUES[str]},
                "some_full_list": [{}, {"name": TEST_VALUES[str]}],
            },
        },
        "id": 7,
    },
}
minimalPullerConf = {
    "LARGE_NESTED_SETTING": {"value": {}, "id": 7},
}
secondOneOfPullerConf = {
    "LARGE_NESTED_SETTING": {"value": {"import": "this"}, "id": 7},
}
impossibleOneOfPullerConf = {
    "LARGE_NESTED_SETTING": {
        "value": {
            #: from first oneOf
            "some_root_field": TEST_VALUES[str],
            #: from second oneOf
            "import": "this",
        },
        "id": 7,
    },
}
# ============= Validation Schemas =====================================================================================
# some of test types (str, int, float, bool) have only two values: the test one and the default one
LUNA_TEST = "luna-test"
TYPE_TEST_STR = {"type": "string", "enum": [TEST_VALUES[str], DEFAULT_VALUES[str]]}
TYPE_TEST_INT = {"type": "integer", "enum": [TEST_VALUES[int], DEFAULT_VALUES[int]]}
TYPE_TEST_FLOAT = {"type": "number", "enum": [TEST_VALUES[float], DEFAULT_VALUES[float]]}
TYPE_TEST_BOOL = {"type": "boolean", "enum": [TEST_VALUES[bool], DEFAULT_VALUES[bool]]}
_APF = ADDITIONAL_PROPERTIES_FALSE = {"additionalProperties": False}
TYPE_TEST_DICT = {
    "type": "object",
    "properties": {"name": TYPE_TEST_STR},
    **_APF,
}
TYPE_EMPTY_LIST = {"type": "array"}
TYPE_TEST_LIST = {"type": "array", "items": TYPE_TEST_DICT}
TYPE_STR_LIST = {"type": "array", "items": TYPE_TEST_STR}
TYPE_INT_LIST = {"type": "array", "items": TYPE_TEST_INT}
TYPE_FLOAT_LIST = {"type": "array", "items": TYPE_TEST_FLOAT}
TYPE_BOOL_LIST = {"type": "array", "items": TYPE_TEST_BOOL}

LARGE_NESTED_SETTING_SCHEMA = {
    "oneOf": [
        {
            "type": "object",
            "properties": {
                "some_root_field": TYPE_TEST_STR,
                "some_data": {
                    "type": "object",
                    "properties": {
                        "some_str": TYPE_TEST_STR,
                        "some_int": TYPE_TEST_INT,
                        "some_float": TYPE_TEST_FLOAT,
                        "some_bool": TYPE_TEST_BOOL,
                        # "some_bool": {"oneOf": [TYPE_TEST_BOOL]}, # todo uncomment after LUNA-2982
                        "some_empty_list": TYPE_EMPTY_LIST,
                        "some_str_list": TYPE_STR_LIST,
                        "some_int_list": TYPE_INT_LIST,
                        "some_float_list": TYPE_FLOAT_LIST,
                        "some_bool_list": TYPE_BOOL_LIST,
                        "some_empty_dict": TYPE_TEST_DICT,
                        "some_full_dict": TYPE_TEST_DICT,
                        "some_full_list": TYPE_TEST_LIST,
                    },
                    **_APF,
                },
            },
            "required": ["some_root_field", "some_data"],
            **_APF,
            "default": {
                "some_root_field": DEFAULT_VALUES[str],
                "some_data": {
                    "some_str": DEFAULT_VALUES[str],
                    "some_int": DEFAULT_VALUES[int],
                    "some_float": DEFAULT_VALUES[float],
                    "some_bool": DEFAULT_VALUES[bool],
                    "some_empty_list": [],
                    "some_str_list": [DEFAULT_VALUES[str]] * 3,
                    "some_int_list": [DEFAULT_VALUES[int]] * 3,
                    "some_float_list": [DEFAULT_VALUES[float]] * 3,
                    "some_bool_list": [DEFAULT_VALUES[bool]] * 3,
                    "some_empty_dict": {},
                    "some_full_dict": {"name": DEFAULT_VALUES[str]},
                    "some_full_list": [{}, {"name": DEFAULT_VALUES[str]}],
                },
            },
        },
        {
            "type": "object",
            "properties": {
                "import": {
                    "type": "string",
                    "enum": ["this"],
                },
            },
            **_APF,
            "default": {
                "import": "this",
            },
        },
    ]
}

# ============= Test 'SETTINGS' Object (Use instead of 'configs.settings.all_settings.SETTINGS') =======================
SETTINGS = {
    "LUNA_CONFIGURATOR": BaseSettingDefinition(
        settingName="LUNA_CONFIGURATOR",
        services=[LUNA_TEST],
        schema=schemas.CONFIGURATOR_USAGE,
        default={
            "use_configurator": False,
            "luna_configurator_origin": "http://127.0.0.1:5070",
            "luna_configurator_api": 1,
        },
        description=schemas.CONFIGURATOR_USAGE["description"],
        groups=[],
    ),
    "LARGE_NESTED_SETTING": BaseSettingDefinition(
        settingName="LARGE_NESTED_SETTING",
        services=[LUNA_TEST],
        schema=LARGE_NESTED_SETTING_SCHEMA,
        default=LARGE_NESTED_SETTING_SCHEMA["oneOf"][0]["default"],
        description="test configs functionality",
        groups=[],
    ),
}


# "luna-test" service settings =========================================================================================


class LargeNestedSettings(BaseSettingClass):
    __slots__ = ("someRootField", "someData", "importThis")

    def __init__(self, serviceName: str, settings: dict):
        super().__init__(serviceName, "LARGE_NESTED_SETTING")

        self.someRootField = settings.get("some_root_field")
        self.someData = settings.get("some_data")
        self.importThis = settings.get("import")


class SettingsTest(BasePythonServiceSettingsClass):
    """
    Test settings class for luna-test.
    Attributes:
        largeNestedSettings: large settings object
    """

    serviceName = "luna-test"
    settingsStore = SETTINGS

    def __init__(self):
        super(BasePythonServiceSettingsClass, self).__init__()
        self.largeNestedSettings = None
        self._inputLargeNestedSettings = None

    def applySettings(self):
        self.largeNestedSettings = LargeNestedSettings(self.serviceName, self["LARGE_NESTED_SETTING"])
        self._inputLargeNestedSettings = self["LARGE_NESTED_SETTING"]


validate(LARGE_NESTED_SETTING_SCHEMA["oneOf"][0], LARGE_NESTED_SETTING_SCHEMA["oneOf"][0]["default"])
validate(LARGE_NESTED_SETTING_SCHEMA["oneOf"][1], LARGE_NESTED_SETTING_SCHEMA["oneOf"][1]["default"])
validate(LARGE_NESTED_SETTING_SCHEMA, LARGE_NESTED_SETTING_SCHEMA["oneOf"][0]["default"])
validate(LARGE_NESTED_SETTING_SCHEMA, LARGE_NESTED_SETTING_SCHEMA["oneOf"][1]["default"])
validate(LARGE_NESTED_SETTING_SCHEMA, pullerConf["LARGE_NESTED_SETTING"]["value"])
